Package: XML
Version: 3.99-0.23
Authors@R: c(person("CRAN Team", role = "ctb",
                    email = "CRAN@r-project.org",
                    comment = "de facto maintainer in 2013-2026"),
             person("Duncan", "Temple Lang", role = "aut",
                    email = "duncan@r-project.org",
                    comment = c(ORCID = "0000-0003-0159-1546")),
             person("Tomas", "Kalibera", role = "ctb"),
             person("Ivan", "Krylov", email = "ikrylov@disroot.org",
                    role = "cre"))
Title: Tools for Parsing and Generating XML Within R and S-Plus
Depends: R (>= 4.0.0), methods, utils
Suggests: bitops, RCurl
SystemRequirements: libxml2 (>= 2.6.3)
Description: Many approaches for both reading and
        creating XML (and HTML) documents (including DTDs), both local
        and accessible via HTTP or FTP.  Also offers access to an
        'XPath' "interpreter".
BugReports: https://codeberg.org/aitap/XML/issues
License: BSD_3_clause + file LICENSE
Collate: AAA.R DTD.R DTDClasses.R DTDRef.R SAXMethods.R XMLClasses.R
        applyDOM.R assignChild.R catalog.R createNode.R dynSupports.R
        error.R flatTree.R nodeAccessors.R parseDTD.R schema.R
        summary.R tangle.R toString.R tree.R version.R xmlErrorEnums.R
        xmlEventHandler.R xmlEventParse.R xmlHandler.R
        xmlInternalSource.R xmlOutputDOM.R xmlNodes.R xmlOutputBuffer.R
        xmlTree.R xmlTreeParse.R htmlParse.R hashTree.R zzz.R
        supports.R parser.R libxmlFeatures.R xmlString.R saveXML.R
        namespaces.R readHTMLTable.R reflection.R xmlToDataFrame.R
        bitList.R compare.R encoding.R fixNS.R xmlRoot.R serialize.R
        xmlMemoryMgmt.R keyValueDB.R solrDocs.R XMLRErrorInfo.R
        xincludes.R namespaceHandlers.R tangle1.R htmlLinks.R
        htmlLists.R getDependencies.R getRelativeURL.R xmlIncludes.R
        simplifyPath.R
NeedsCompilation: yes
Packaged: 2026-03-19 18:59:15 UTC; aitap
Author: CRAN Team [ctb] (de facto maintainer in 2013-2026),
  Duncan Temple Lang [aut] (ORCID:
    <https://orcid.org/0000-0003-0159-1546>),
  Tomas Kalibera [ctb],
  Ivan Krylov [cre]
Maintainer: Ivan Krylov <ikrylov@disroot.org>
Repository: CRAN
Date/Publication: 2026-03-20 06:11:24 UTC
