
dataFrameEvent.R     example of closure for reading a data frame via the event based parser.
foo.dtd              DTD used for testing the DTD parsing facilities (getDTD()).
job.xml              example XML file taken from the libxml-1.7.3 examples.
enhancedDataFrame.R  version of the event-driven data frame reader with more error checking.


Some SVG files are available from http://plasma-gate.weizmann.ac.il/Grace/gallery/