The Packages/ directory has some package source tar.gz files.

See index.html for a description of the package and the installation
procedures.

This R package is not in the R package format in the github repository.
It was initially developed in 1999 and was intended for use in both
S-Plus and R and so requires a different structure for each.

    make ADMIN=1 

copies the files to an appropriate structure for R.
It currently requires some supporting tools from the Omegahat
admin facilities.










