dtdIsAttribute <- function(name, element, dtd) .Defunct(package = 'XML')
dtdValidElement <- function(name, within, dtd, pos=NULL) .Defunct(package = 'XML')
dtdElementValidEntry <- function(element, name, pos=NULL) .Defunct(package = 'XML')
xmlContainsEntity <- function(name, dtd) .Defunct(package = 'XML')
xmlContainsElement <- function(name, dtd) .Defunct(package = 'XML')
dtdEntity <- function(name, dtd) .Defunct(package = 'XML')
dtdElement <- function(name, dtd) .Defunct(package = 'XML')
parseDTD <- function(
	extId, asText = FALSE, name = "", isURL = FALSE, error = xmlErrorCumulator()
) .Defunct(package = 'XML')

