parseDTD <- 
function(extId, asText = FALSE, name = "", isURL = FALSE, error = xmlErrorCumulator())
{
  .Deprecated(msg = "'parseDTD' no longer works, sorry.", package = 'XML')
  structure(list(), class = c('InternalDTD', 'ExternalDTD'))
}
