# Machine generated file. See org/omegahat/XML/RS/TU/tu.R
# Wed Feb 13 03:17:42 2008 
xmlErrorLevel <-structure(0:3, .Names = c("XML_ERR_NONE", "XML_ERR_WARNING", 
"XML_ERR_ERROR", "XML_ERR_FATAL"))
xmlErrorDomain <-structure(0:27, .Names = c("XML_FROM_NONE", "XML_FROM_PARSER", 
"XML_FROM_TREE", "XML_FROM_NAMESPACE", "XML_FROM_DTD", "XML_FROM_HTML", 
"XML_FROM_MEMORY", "XML_FROM_OUTPUT", "XML_FROM_IO", "XML_FROM_FTP", 
"XML_FROM_HTTP", "XML_FROM_XINCLUDE", "XML_FROM_XPATH", "XML_FROM_XPOINTER", 
"XML_FROM_REGEXP", "XML_FROM_DATATYPE", "XML_FROM_SCHEMASP", 
"XML_FROM_SCHEMASV", "XML_FROM_RELAXNGP", "XML_FROM_RELAXNGV", 
"XML_FROM_CATALOG", "XML_FROM_C14N", "XML_FROM_XSLT", "XML_FROM_VALID", 
"XML_FROM_CHECK", "XML_FROM_WRITER", "XML_FROM_MODULE", "XML_FROM_I18N"
))
xmlParserErrors <-structure(c(0L, 1L, 2L, 3L, 4L, 5L, 6L, 7L, 8L, 9L, 10L, 11L, 
12L, 13L, 14L, 15L, 16L, 17L, 18L, 19L, 20L, 21L, 22L, 23L, 24L, 
25L, 26L, 27L, 28L, 29L, 30L, 31L, 32L, 33L, 34L, 35L, 36L, 37L, 
38L, 39L, 40L, 41L, 42L, 43L, 44L, 45L, 46L, 47L, 48L, 49L, 50L, 
51L, 52L, 53L, 54L, 55L, 56L, 57L, 58L, 59L, 60L, 61L, 62L, 63L, 
64L, 65L, 66L, 67L, 68L, 69L, 70L, 71L, 72L, 73L, 74L, 75L, 76L, 
77L, 78L, 79L, 80L, 81L, 82L, 83L, 84L, 85L, 86L, 87L, 88L, 89L, 
90L, 91L, 92L, 93L, 94L, 95L, 96L, 97L, 98L, 99L, 100L, 101L, 
102L, 103L, 104L, 105L, 106L, 107L, 200L, 201L, 202L, 203L, 204L, 
500L, 501L, 502L, 503L, 504L, 505L, 506L, 507L, 508L, 509L, 510L, 
511L, 512L, 513L, 514L, 515L, 516L, 517L, 518L, 519L, 520L, 521L, 
522L, 523L, 524L, 525L, 526L, 527L, 528L, 529L, 530L, 531L, 532L, 
533L, 534L, 535L, 536L, 537L, 538L, 539L, 540L, 800L, 801L, 1000L, 
1001L, 1002L, 1003L, 1004L, 1005L, 1006L, 1007L, 1008L, 1009L, 
1010L, 1011L, 1012L, 1013L, 1014L, 1015L, 1016L, 1017L, 1018L, 
1019L, 1020L, 1021L, 1022L, 1023L, 1024L, 1025L, 1026L, 1027L, 
1028L, 1029L, 1030L, 1031L, 1032L, 1033L, 1034L, 1035L, 1036L, 
1037L, 1038L, 1039L, 1040L, 1041L, 1042L, 1043L, 1044L, 1045L, 
1046L, 1047L, 1048L, 1049L, 1050L, 1051L, 1052L, 1053L, 1054L, 
1055L, 1056L, 1057L, 1058L, 1059L, 1060L, 1061L, 1062L, 1063L, 
1064L, 1065L, 1066L, 1067L, 1068L, 1069L, 1070L, 1071L, 1072L, 
1073L, 1074L, 1075L, 1076L, 1077L, 1078L, 1079L, 1080L, 1081L, 
1082L, 1083L, 1084L, 1085L, 1086L, 1087L, 1088L, 1089L, 1090L, 
1091L, 1092L, 1093L, 1094L, 1095L, 1096L, 1097L, 1098L, 1099L, 
1100L, 1101L, 1102L, 1103L, 1104L, 1105L, 1106L, 1107L, 1108L, 
1109L, 1110L, 1111L, 1112L, 1113L, 1114L, 1115L, 1116L, 1117L, 
1118L, 1119L, 1120L, 1121L, 1122L, 1200L, 1201L, 1202L, 1203L, 
1204L, 1205L, 1206L, 1207L, 1208L, 1209L, 1210L, 1211L, 1212L, 
1213L, 1214L, 1215L, 1216L, 1217L, 1218L, 1219L, 1220L, 1221L, 
1300L, 1301L, 1302L, 1400L, 1401L, 1402L, 1403L, 1450L, 1500L, 
1501L, 1502L, 1503L, 1504L, 1505L, 1506L, 1507L, 1508L, 1509L, 
1510L, 1511L, 1512L, 1513L, 1514L, 1515L, 1516L, 1517L, 1518L, 
1519L, 1520L, 1521L, 1522L, 1523L, 1524L, 1525L, 1526L, 1527L, 
1528L, 1529L, 1530L, 1531L, 1532L, 1533L, 1534L, 1535L, 1536L, 
1537L, 1538L, 1539L, 1540L, 1541L, 1542L, 1543L, 1544L, 1545L, 
1546L, 1547L, 1548L, 1549L, 1550L, 1551L, 1552L, 1553L, 1554L, 
1555L, 1556L, 1600L, 1601L, 1602L, 1603L, 1604L, 1605L, 1606L, 
1607L, 1608L, 1609L, 1610L, 1611L, 1612L, 1613L, 1614L, 1615L, 
1616L, 1617L, 1618L, 1650L, 1651L, 1652L, 1653L, 1654L, 1700L, 
1701L, 1702L, 1703L, 1704L, 1705L, 1706L, 1707L, 1708L, 1709L, 
1710L, 1711L, 1712L, 1713L, 1714L, 1715L, 1716L, 1717L, 1718L, 
1719L, 1720L, 1721L, 1722L, 1723L, 1724L, 1725L, 1726L, 1727L, 
1728L, 1729L, 1730L, 1731L, 1732L, 1733L, 1734L, 1735L, 1736L, 
1737L, 1738L, 1739L, 1740L, 1741L, 1742L, 1743L, 1744L, 1745L, 
1746L, 1747L, 1748L, 1749L, 1750L, 1751L, 1752L, 1753L, 1754L, 
1755L, 1756L, 1757L, 1758L, 1759L, 1760L, 1761L, 1762L, 1763L, 
1764L, 1765L, 1766L, 1767L, 1768L, 1769L, 1770L, 1771L, 1772L, 
1773L, 1774L, 1775L, 1776L, 1777L, 1778L, 1779L, 1780L, 1781L, 
1782L, 1783L, 1784L, 1785L, 1786L, 1787L, 1788L, 1789L, 1790L, 
1791L, 1792L, 1793L, 1794L, 1795L, 1796L, 1797L, 1798L, 1799L, 
1800L, 1801L, 1802L, 1803L, 1804L, 1805L, 1806L, 1807L, 1808L, 
1809L, 1810L, 1811L, 1812L, 1813L, 1814L, 1815L, 1816L, 1817L, 
1818L, 1819L, 1820L, 1821L, 1822L, 1823L, 1824L, 1825L, 1826L, 
1827L, 1828L, 1829L, 1830L, 1831L, 1832L, 1833L, 1834L, 1835L, 
1836L, 1837L, 1838L, 1839L, 1840L, 1841L, 1842L, 1843L, 1844L, 
1845L, 1846L, 1847L, 1848L, 1849L, 1850L, 1851L, 1852L, 1853L, 
1854L, 1855L, 1856L, 1857L, 1858L, 1859L, 1860L, 1861L, 1862L, 
1863L, 1864L, 1865L, 1866L, 1867L, 1868L, 1869L, 1870L, 1871L, 
1872L, 1873L, 1874L, 1875L, 1876L, 1877L, 1878L, 1879L, 1900L, 
1901L, 1902L, 1903L, 1950L, 1951L, 1952L, 1953L, 1954L, 1955L, 
2000L, 2001L, 2002L, 2003L, 2020L, 2021L, 2022L, 3000L, 3001L, 
3002L, 3003L, 3004L, 3005L, 3006L, 3007L, 3008L, 3009L, 3010L, 
3011L, 3012L, 3013L, 3014L, 3015L, 3016L, 3017L, 3018L, 3019L, 
3020L, 3021L, 3022L, 3023L, 3024L, 3025L, 3026L, 3027L, 3028L, 
3029L, 3030L, 3031L, 3032L, 3033L, 3034L, 3035L, 3036L, 3037L, 
3038L, 3039L, 3040L, 3041L, 3042L, 3043L, 3044L, 3045L, 3046L, 
3047L, 3048L, 3049L, 3050L, 3051L, 3052L, 3053L, 3054L, 3055L, 
3056L, 3057L, 3058L, 3059L, 3060L, 3061L, 3062L, 3063L, 3064L, 
3065L, 3066L, 3067L, 3068L, 3069L, 3070L, 3071L, 3072L, 3073L, 
3074L, 3075L, 3076L, 3077L, 3078L, 3079L, 3080L, 3081L, 3082L, 
3083L, 3084L, 3085L, 3086L, 3087L, 3088L, 3089L, 3090L, 3091L, 
4900L, 4901L, 5000L, 5001L, 5002L, 5003L, 5004L, 5005L, 5006L, 
5007L, 5008L, 5009L, 5010L, 5011L, 5012L, 5013L, 5014L, 5015L, 
5016L, 5017L, 5018L, 5019L, 5020L, 5021L, 5022L, 5023L, 5024L, 
5025L, 5026L, 5027L, 5028L, 5029L, 5030L, 5031L, 5032L, 5033L, 
5034L, 5035L, 5036L, 5037L, 6000L, 6001L, 6002L, 6003L, 6004L
), .Names = c("XML_ERR_OK", "XML_ERR_INTERNAL_ERROR", "XML_ERR_NO_MEMORY", 
"XML_ERR_DOCUMENT_START", "XML_ERR_DOCUMENT_EMPTY", "XML_ERR_DOCUMENT_END", 
"XML_ERR_INVALID_HEX_CHARREF", "XML_ERR_INVALID_DEC_CHARREF", 
"XML_ERR_INVALID_CHARREF", "XML_ERR_INVALID_CHAR", "XML_ERR_CHARREF_AT_EOF", 
"XML_ERR_CHARREF_IN_PROLOG", "XML_ERR_CHARREF_IN_EPILOG", "XML_ERR_CHARREF_IN_DTD", 
"XML_ERR_ENTITYREF_AT_EOF", "XML_ERR_ENTITYREF_IN_PROLOG", "XML_ERR_ENTITYREF_IN_EPILOG", 
"XML_ERR_ENTITYREF_IN_DTD", "XML_ERR_PEREF_AT_EOF", "XML_ERR_PEREF_IN_PROLOG", 
"XML_ERR_PEREF_IN_EPILOG", "XML_ERR_PEREF_IN_INT_SUBSET", "XML_ERR_ENTITYREF_NO_NAME", 
"XML_ERR_ENTITYREF_SEMICOL_MISSING", "XML_ERR_PEREF_NO_NAME", 
"XML_ERR_PEREF_SEMICOL_MISSING", "XML_ERR_UNDECLARED_ENTITY", 
"XML_WAR_UNDECLARED_ENTITY", "XML_ERR_UNPARSED_ENTITY", "XML_ERR_ENTITY_IS_EXTERNAL", 
"XML_ERR_ENTITY_IS_PARAMETER", "XML_ERR_UNKNOWN_ENCODING", "XML_ERR_UNSUPPORTED_ENCODING", 
"XML_ERR_STRING_NOT_STARTED", "XML_ERR_STRING_NOT_CLOSED", "XML_ERR_NS_DECL_ERROR", 
"XML_ERR_ENTITY_NOT_STARTED", "XML_ERR_ENTITY_NOT_FINISHED", 
"XML_ERR_LT_IN_ATTRIBUTE", "XML_ERR_ATTRIBUTE_NOT_STARTED", "XML_ERR_ATTRIBUTE_NOT_FINISHED", 
"XML_ERR_ATTRIBUTE_WITHOUT_VALUE", "XML_ERR_ATTRIBUTE_REDEFINED", 
"XML_ERR_LITERAL_NOT_STARTED", "XML_ERR_LITERAL_NOT_FINISHED", 
"XML_ERR_COMMENT_NOT_FINISHED", "XML_ERR_PI_NOT_STARTED", "XML_ERR_PI_NOT_FINISHED", 
"XML_ERR_NOTATION_NOT_STARTED", "XML_ERR_NOTATION_NOT_FINISHED", 
"XML_ERR_ATTLIST_NOT_STARTED", "XML_ERR_ATTLIST_NOT_FINISHED", 
"XML_ERR_MIXED_NOT_STARTED", "XML_ERR_MIXED_NOT_FINISHED", "XML_ERR_ELEMCONTENT_NOT_STARTED", 
"XML_ERR_ELEMCONTENT_NOT_FINISHED", "XML_ERR_XMLDECL_NOT_STARTED", 
"XML_ERR_XMLDECL_NOT_FINISHED", "XML_ERR_CONDSEC_NOT_STARTED", 
"XML_ERR_CONDSEC_NOT_FINISHED", "XML_ERR_EXT_SUBSET_NOT_FINISHED", 
"XML_ERR_DOCTYPE_NOT_FINISHED", "XML_ERR_MISPLACED_CDATA_END", 
"XML_ERR_CDATA_NOT_FINISHED", "XML_ERR_RESERVED_XML_NAME", "XML_ERR_SPACE_REQUIRED", 
"XML_ERR_SEPARATOR_REQUIRED", "XML_ERR_NMTOKEN_REQUIRED", "XML_ERR_NAME_REQUIRED", 
"XML_ERR_PCDATA_REQUIRED", "XML_ERR_URI_REQUIRED", "XML_ERR_PUBID_REQUIRED", 
"XML_ERR_LT_REQUIRED", "XML_ERR_GT_REQUIRED", "XML_ERR_LTSLASH_REQUIRED", 
"XML_ERR_EQUAL_REQUIRED", "XML_ERR_TAG_NAME_MISMATCH", "XML_ERR_TAG_NOT_FINISHED", 
"XML_ERR_STANDALONE_VALUE", "XML_ERR_ENCODING_NAME", "XML_ERR_HYPHEN_IN_COMMENT", 
"XML_ERR_INVALID_ENCODING", "XML_ERR_EXT_ENTITY_STANDALONE", 
"XML_ERR_CONDSEC_INVALID", "XML_ERR_VALUE_REQUIRED", "XML_ERR_NOT_WELL_BALANCED", 
"XML_ERR_EXTRA_CONTENT", "XML_ERR_ENTITY_CHAR_ERROR", "XML_ERR_ENTITY_PE_INTERNAL", 
"XML_ERR_ENTITY_LOOP", "XML_ERR_ENTITY_BOUNDARY", "XML_ERR_INVALID_URI", 
"XML_ERR_URI_FRAGMENT", "XML_WAR_CATALOG_PI", "XML_ERR_NO_DTD", 
"XML_ERR_CONDSEC_INVALID_KEYWORD", "XML_ERR_VERSION_MISSING", 
"XML_WAR_UNKNOWN_VERSION", "XML_WAR_LANG_VALUE", "XML_WAR_NS_URI", 
"XML_WAR_NS_URI_RELATIVE", "XML_ERR_MISSING_ENCODING", "XML_WAR_SPACE_VALUE", 
"XML_ERR_NOT_STANDALONE", "XML_ERR_ENTITY_PROCESSING", "XML_ERR_NOTATION_PROCESSING", 
"XML_WAR_NS_COLUMN", "XML_WAR_ENTITY_REDEFINED", "XML_NS_ERR_XML_NAMESPACE", 
"XML_NS_ERR_UNDEFINED_NAMESPACE", "XML_NS_ERR_QNAME", "XML_NS_ERR_ATTRIBUTE_REDEFINED", 
"XML_NS_ERR_EMPTY", "XML_DTD_ATTRIBUTE_DEFAULT", "XML_DTD_ATTRIBUTE_REDEFINED", 
"XML_DTD_ATTRIBUTE_VALUE", "XML_DTD_CONTENT_ERROR", "XML_DTD_CONTENT_MODEL", 
"XML_DTD_CONTENT_NOT_DETERMINIST", "XML_DTD_DIFFERENT_PREFIX", 
"XML_DTD_ELEM_DEFAULT_NAMESPACE", "XML_DTD_ELEM_NAMESPACE", "XML_DTD_ELEM_REDEFINED", 
"XML_DTD_EMPTY_NOTATION", "XML_DTD_ENTITY_TYPE", "XML_DTD_ID_FIXED", 
"XML_DTD_ID_REDEFINED", "XML_DTD_ID_SUBSET", "XML_DTD_INVALID_CHILD", 
"XML_DTD_INVALID_DEFAULT", "XML_DTD_LOAD_ERROR", "XML_DTD_MISSING_ATTRIBUTE", 
"XML_DTD_MIXED_CORRUPT", "XML_DTD_MULTIPLE_ID", "XML_DTD_NO_DOC", 
"XML_DTD_NO_DTD", "XML_DTD_NO_ELEM_NAME", "XML_DTD_NO_PREFIX", 
"XML_DTD_NO_ROOT", "XML_DTD_NOTATION_REDEFINED", "XML_DTD_NOTATION_VALUE", 
"XML_DTD_NOT_EMPTY", "XML_DTD_NOT_PCDATA", "XML_DTD_NOT_STANDALONE", 
"XML_DTD_ROOT_NAME", "XML_DTD_STANDALONE_WHITE_SPACE", "XML_DTD_UNKNOWN_ATTRIBUTE", 
"XML_DTD_UNKNOWN_ELEM", "XML_DTD_UNKNOWN_ENTITY", "XML_DTD_UNKNOWN_ID", 
"XML_DTD_UNKNOWN_NOTATION", "XML_DTD_STANDALONE_DEFAULTED", "XML_DTD_XMLID_VALUE", 
"XML_DTD_XMLID_TYPE", "XML_HTML_STRUCURE_ERROR", "XML_HTML_UNKNOWN_TAG", 
"XML_RNGP_ANYNAME_ATTR_ANCESTOR", "XML_RNGP_ATTR_CONFLICT", "XML_RNGP_ATTRIBUTE_CHILDREN", 
"XML_RNGP_ATTRIBUTE_CONTENT", "XML_RNGP_ATTRIBUTE_EMPTY", "XML_RNGP_ATTRIBUTE_NOOP", 
"XML_RNGP_CHOICE_CONTENT", "XML_RNGP_CHOICE_EMPTY", "XML_RNGP_CREATE_FAILURE", 
"XML_RNGP_DATA_CONTENT", "XML_RNGP_DEF_CHOICE_AND_INTERLEAVE", 
"XML_RNGP_DEFINE_CREATE_FAILED", "XML_RNGP_DEFINE_EMPTY", "XML_RNGP_DEFINE_MISSING", 
"XML_RNGP_DEFINE_NAME_MISSING", "XML_RNGP_ELEM_CONTENT_EMPTY", 
"XML_RNGP_ELEM_CONTENT_ERROR", "XML_RNGP_ELEMENT_EMPTY", "XML_RNGP_ELEMENT_CONTENT", 
"XML_RNGP_ELEMENT_NAME", "XML_RNGP_ELEMENT_NO_CONTENT", "XML_RNGP_ELEM_TEXT_CONFLICT", 
"XML_RNGP_EMPTY", "XML_RNGP_EMPTY_CONSTRUCT", "XML_RNGP_EMPTY_CONTENT", 
"XML_RNGP_EMPTY_NOT_EMPTY", "XML_RNGP_ERROR_TYPE_LIB", "XML_RNGP_EXCEPT_EMPTY", 
"XML_RNGP_EXCEPT_MISSING", "XML_RNGP_EXCEPT_MULTIPLE", "XML_RNGP_EXCEPT_NO_CONTENT", 
"XML_RNGP_EXTERNALREF_EMTPY", "XML_RNGP_EXTERNAL_REF_FAILURE", 
"XML_RNGP_EXTERNALREF_RECURSE", "XML_RNGP_FORBIDDEN_ATTRIBUTE", 
"XML_RNGP_FOREIGN_ELEMENT", "XML_RNGP_GRAMMAR_CONTENT", "XML_RNGP_GRAMMAR_EMPTY", 
"XML_RNGP_GRAMMAR_MISSING", "XML_RNGP_GRAMMAR_NO_START", "XML_RNGP_GROUP_ATTR_CONFLICT", 
"XML_RNGP_HREF_ERROR", "XML_RNGP_INCLUDE_EMPTY", "XML_RNGP_INCLUDE_FAILURE", 
"XML_RNGP_INCLUDE_RECURSE", "XML_RNGP_INTERLEAVE_ADD", "XML_RNGP_INTERLEAVE_CREATE_FAILED", 
"XML_RNGP_INTERLEAVE_EMPTY", "XML_RNGP_INTERLEAVE_NO_CONTENT", 
"XML_RNGP_INVALID_DEFINE_NAME", "XML_RNGP_INVALID_URI", "XML_RNGP_INVALID_VALUE", 
"XML_RNGP_MISSING_HREF", "XML_RNGP_NAME_MISSING", "XML_RNGP_NEED_COMBINE", 
"XML_RNGP_NOTALLOWED_NOT_EMPTY", "XML_RNGP_NSNAME_ATTR_ANCESTOR", 
"XML_RNGP_NSNAME_NO_NS", "XML_RNGP_PARAM_FORBIDDEN", "XML_RNGP_PARAM_NAME_MISSING", 
"XML_RNGP_PARENTREF_CREATE_FAILED", "XML_RNGP_PARENTREF_NAME_INVALID", 
"XML_RNGP_PARENTREF_NO_NAME", "XML_RNGP_PARENTREF_NO_PARENT", 
"XML_RNGP_PARENTREF_NOT_EMPTY", "XML_RNGP_PARSE_ERROR", "XML_RNGP_PAT_ANYNAME_EXCEPT_ANYNAME", 
"XML_RNGP_PAT_ATTR_ATTR", "XML_RNGP_PAT_ATTR_ELEM", "XML_RNGP_PAT_DATA_EXCEPT_ATTR", 
"XML_RNGP_PAT_DATA_EXCEPT_ELEM", "XML_RNGP_PAT_DATA_EXCEPT_EMPTY", 
"XML_RNGP_PAT_DATA_EXCEPT_GROUP", "XML_RNGP_PAT_DATA_EXCEPT_INTERLEAVE", 
"XML_RNGP_PAT_DATA_EXCEPT_LIST", "XML_RNGP_PAT_DATA_EXCEPT_ONEMORE", 
"XML_RNGP_PAT_DATA_EXCEPT_REF", "XML_RNGP_PAT_DATA_EXCEPT_TEXT", 
"XML_RNGP_PAT_LIST_ATTR", "XML_RNGP_PAT_LIST_ELEM", "XML_RNGP_PAT_LIST_INTERLEAVE", 
"XML_RNGP_PAT_LIST_LIST", "XML_RNGP_PAT_LIST_REF", "XML_RNGP_PAT_LIST_TEXT", 
"XML_RNGP_PAT_NSNAME_EXCEPT_ANYNAME", "XML_RNGP_PAT_NSNAME_EXCEPT_NSNAME", 
"XML_RNGP_PAT_ONEMORE_GROUP_ATTR", "XML_RNGP_PAT_ONEMORE_INTERLEAVE_ATTR", 
"XML_RNGP_PAT_START_ATTR", "XML_RNGP_PAT_START_DATA", "XML_RNGP_PAT_START_EMPTY", 
"XML_RNGP_PAT_START_GROUP", "XML_RNGP_PAT_START_INTERLEAVE", 
"XML_RNGP_PAT_START_LIST", "XML_RNGP_PAT_START_ONEMORE", "XML_RNGP_PAT_START_TEXT", 
"XML_RNGP_PAT_START_VALUE", "XML_RNGP_PREFIX_UNDEFINED", "XML_RNGP_REF_CREATE_FAILED", 
"XML_RNGP_REF_CYCLE", "XML_RNGP_REF_NAME_INVALID", "XML_RNGP_REF_NO_DEF", 
"XML_RNGP_REF_NO_NAME", "XML_RNGP_REF_NOT_EMPTY", "XML_RNGP_START_CHOICE_AND_INTERLEAVE", 
"XML_RNGP_START_CONTENT", "XML_RNGP_START_EMPTY", "XML_RNGP_START_MISSING", 
"XML_RNGP_TEXT_EXPECTED", "XML_RNGP_TEXT_HAS_CHILD", "XML_RNGP_TYPE_MISSING", 
"XML_RNGP_TYPE_NOT_FOUND", "XML_RNGP_TYPE_VALUE", "XML_RNGP_UNKNOWN_ATTRIBUTE", 
"XML_RNGP_UNKNOWN_COMBINE", "XML_RNGP_UNKNOWN_CONSTRUCT", "XML_RNGP_UNKNOWN_TYPE_LIB", 
"XML_RNGP_URI_FRAGMENT", "XML_RNGP_URI_NOT_ABSOLUTE", "XML_RNGP_VALUE_EMPTY", 
"XML_RNGP_VALUE_NO_CONTENT", "XML_RNGP_XMLNS_NAME", "XML_RNGP_XML_NS", 
"XML_XPATH_EXPRESSION_OK", "XML_XPATH_NUMBER_ERROR", "XML_XPATH_UNFINISHED_LITERAL_ERROR", 
"XML_XPATH_START_LITERAL_ERROR", "XML_XPATH_VARIABLE_REF_ERROR", 
"XML_XPATH_UNDEF_VARIABLE_ERROR", "XML_XPATH_INVALID_PREDICATE_ERROR", 
"XML_XPATH_EXPR_ERROR", "XML_XPATH_UNCLOSED_ERROR", "XML_XPATH_UNKNOWN_FUNC_ERROR", 
"XML_XPATH_INVALID_OPERAND", "XML_XPATH_INVALID_TYPE", "XML_XPATH_INVALID_ARITY", 
"XML_XPATH_INVALID_CTXT_SIZE", "XML_XPATH_INVALID_CTXT_POSITION", 
"XML_XPATH_MEMORY_ERROR", "XML_XPTR_SYNTAX_ERROR", "XML_XPTR_RESOURCE_ERROR", 
"XML_XPTR_SUB_RESOURCE_ERROR", "XML_XPATH_UNDEF_PREFIX_ERROR", 
"XML_XPATH_ENCODING_ERROR", "XML_XPATH_INVALID_CHAR_ERROR", "XML_TREE_INVALID_HEX", 
"XML_TREE_INVALID_DEC", "XML_TREE_UNTERMINATED_ENTITY", "XML_SAVE_NOT_UTF8", 
"XML_SAVE_CHAR_INVALID", "XML_SAVE_NO_DOCTYPE", "XML_SAVE_UNKNOWN_ENCODING", 
"XML_REGEXP_COMPILE_ERROR", "XML_IO_UNKNOWN", "XML_IO_EACCES", 
"XML_IO_EAGAIN", "XML_IO_EBADF", "XML_IO_EBADMSG", "XML_IO_EBUSY", 
"XML_IO_ECANCELED", "XML_IO_ECHILD", "XML_IO_EDEADLK", "XML_IO_EDOM", 
"XML_IO_EEXIST", "XML_IO_EFAULT", "XML_IO_EFBIG", "XML_IO_EINPROGRESS", 
"XML_IO_EINTR", "XML_IO_EINVAL", "XML_IO_EIO", "XML_IO_EISDIR", 
"XML_IO_EMFILE", "XML_IO_EMLINK", "XML_IO_EMSGSIZE", "XML_IO_ENAMETOOLONG", 
"XML_IO_ENFILE", "XML_IO_ENODEV", "XML_IO_ENOENT", "XML_IO_ENOEXEC", 
"XML_IO_ENOLCK", "XML_IO_ENOMEM", "XML_IO_ENOSPC", "XML_IO_ENOSYS", 
"XML_IO_ENOTDIR", "XML_IO_ENOTEMPTY", "XML_IO_ENOTSUP", "XML_IO_ENOTTY", 
"XML_IO_ENXIO", "XML_IO_EPERM", "XML_IO_EPIPE", "XML_IO_ERANGE", 
"XML_IO_EROFS", "XML_IO_ESPIPE", "XML_IO_ESRCH", "XML_IO_ETIMEDOUT", 
"XML_IO_EXDEV", "XML_IO_NETWORK_ATTEMPT", "XML_IO_ENCODER", "XML_IO_FLUSH", 
"XML_IO_WRITE", "XML_IO_NO_INPUT", "XML_IO_BUFFER_FULL", "XML_IO_LOAD_ERROR", 
"XML_IO_ENOTSOCK", "XML_IO_EISCONN", "XML_IO_ECONNREFUSED", "XML_IO_ENETUNREACH", 
"XML_IO_EADDRINUSE", "XML_IO_EALREADY", "XML_IO_EAFNOSUPPORT", 
"XML_XINCLUDE_RECURSION", "XML_XINCLUDE_PARSE_VALUE", "XML_XINCLUDE_ENTITY_DEF_MISMATCH", 
"XML_XINCLUDE_NO_HREF", "XML_XINCLUDE_NO_FALLBACK", "XML_XINCLUDE_HREF_URI", 
"XML_XINCLUDE_TEXT_FRAGMENT", "XML_XINCLUDE_TEXT_DOCUMENT", "XML_XINCLUDE_INVALID_CHAR", 
"XML_XINCLUDE_BUILD_FAILED", "XML_XINCLUDE_UNKNOWN_ENCODING", 
"XML_XINCLUDE_MULTIPLE_ROOT", "XML_XINCLUDE_XPTR_FAILED", "XML_XINCLUDE_XPTR_RESULT", 
"XML_XINCLUDE_INCLUDE_IN_INCLUDE", "XML_XINCLUDE_FALLBACKS_IN_INCLUDE", 
"XML_XINCLUDE_FALLBACK_NOT_IN_INCLUDE", "XML_XINCLUDE_DEPRECATED_NS", 
"XML_XINCLUDE_FRAGMENT_ID", "XML_CATALOG_MISSING_ATTR", "XML_CATALOG_ENTRY_BROKEN", 
"XML_CATALOG_PREFER_VALUE", "XML_CATALOG_NOT_CATALOG", "XML_CATALOG_RECURSION", 
"XML_SCHEMAP_PREFIX_UNDEFINED", "XML_SCHEMAP_ATTRFORMDEFAULT_VALUE", 
"XML_SCHEMAP_ATTRGRP_NONAME_NOREF", "XML_SCHEMAP_ATTR_NONAME_NOREF", 
"XML_SCHEMAP_COMPLEXTYPE_NONAME_NOREF", "XML_SCHEMAP_ELEMFORMDEFAULT_VALUE", 
"XML_SCHEMAP_ELEM_NONAME_NOREF", "XML_SCHEMAP_EXTENSION_NO_BASE", 
"XML_SCHEMAP_FACET_NO_VALUE", "XML_SCHEMAP_FAILED_BUILD_IMPORT", 
"XML_SCHEMAP_GROUP_NONAME_NOREF", "XML_SCHEMAP_IMPORT_NAMESPACE_NOT_URI", 
"XML_SCHEMAP_IMPORT_REDEFINE_NSNAME", "XML_SCHEMAP_IMPORT_SCHEMA_NOT_URI", 
"XML_SCHEMAP_INVALID_BOOLEAN", "XML_SCHEMAP_INVALID_ENUM", "XML_SCHEMAP_INVALID_FACET", 
"XML_SCHEMAP_INVALID_FACET_VALUE", "XML_SCHEMAP_INVALID_MAXOCCURS", 
"XML_SCHEMAP_INVALID_MINOCCURS", "XML_SCHEMAP_INVALID_REF_AND_SUBTYPE", 
"XML_SCHEMAP_INVALID_WHITE_SPACE", "XML_SCHEMAP_NOATTR_NOREF", 
"XML_SCHEMAP_NOTATION_NO_NAME", "XML_SCHEMAP_NOTYPE_NOREF", "XML_SCHEMAP_REF_AND_SUBTYPE", 
"XML_SCHEMAP_RESTRICTION_NONAME_NOREF", "XML_SCHEMAP_SIMPLETYPE_NONAME", 
"XML_SCHEMAP_TYPE_AND_SUBTYPE", "XML_SCHEMAP_UNKNOWN_ALL_CHILD", 
"XML_SCHEMAP_UNKNOWN_ANYATTRIBUTE_CHILD", "XML_SCHEMAP_UNKNOWN_ATTR_CHILD", 
"XML_SCHEMAP_UNKNOWN_ATTRGRP_CHILD", "XML_SCHEMAP_UNKNOWN_ATTRIBUTE_GROUP", 
"XML_SCHEMAP_UNKNOWN_BASE_TYPE", "XML_SCHEMAP_UNKNOWN_CHOICE_CHILD", 
"XML_SCHEMAP_UNKNOWN_COMPLEXCONTENT_CHILD", "XML_SCHEMAP_UNKNOWN_COMPLEXTYPE_CHILD", 
"XML_SCHEMAP_UNKNOWN_ELEM_CHILD", "XML_SCHEMAP_UNKNOWN_EXTENSION_CHILD", 
"XML_SCHEMAP_UNKNOWN_FACET_CHILD", "XML_SCHEMAP_UNKNOWN_FACET_TYPE", 
"XML_SCHEMAP_UNKNOWN_GROUP_CHILD", "XML_SCHEMAP_UNKNOWN_IMPORT_CHILD", 
"XML_SCHEMAP_UNKNOWN_LIST_CHILD", "XML_SCHEMAP_UNKNOWN_NOTATION_CHILD", 
"XML_SCHEMAP_UNKNOWN_PROCESSCONTENT_CHILD", "XML_SCHEMAP_UNKNOWN_REF", 
"XML_SCHEMAP_UNKNOWN_RESTRICTION_CHILD", "XML_SCHEMAP_UNKNOWN_SCHEMAS_CHILD", 
"XML_SCHEMAP_UNKNOWN_SEQUENCE_CHILD", "XML_SCHEMAP_UNKNOWN_SIMPLECONTENT_CHILD", 
"XML_SCHEMAP_UNKNOWN_SIMPLETYPE_CHILD", "XML_SCHEMAP_UNKNOWN_TYPE", 
"XML_SCHEMAP_UNKNOWN_UNION_CHILD", "XML_SCHEMAP_ELEM_DEFAULT_FIXED", 
"XML_SCHEMAP_REGEXP_INVALID", "XML_SCHEMAP_FAILED_LOAD", "XML_SCHEMAP_NOTHING_TO_PARSE", 
"XML_SCHEMAP_NOROOT", "XML_SCHEMAP_REDEFINED_GROUP", "XML_SCHEMAP_REDEFINED_TYPE", 
"XML_SCHEMAP_REDEFINED_ELEMENT", "XML_SCHEMAP_REDEFINED_ATTRGROUP", 
"XML_SCHEMAP_REDEFINED_ATTR", "XML_SCHEMAP_REDEFINED_NOTATION", 
"XML_SCHEMAP_FAILED_PARSE", "XML_SCHEMAP_UNKNOWN_PREFIX", "XML_SCHEMAP_DEF_AND_PREFIX", 
"XML_SCHEMAP_UNKNOWN_INCLUDE_CHILD", "XML_SCHEMAP_INCLUDE_SCHEMA_NOT_URI", 
"XML_SCHEMAP_INCLUDE_SCHEMA_NO_URI", "XML_SCHEMAP_NOT_SCHEMA", 
"XML_SCHEMAP_UNKNOWN_MEMBER_TYPE", "XML_SCHEMAP_INVALID_ATTR_USE", 
"XML_SCHEMAP_RECURSIVE", "XML_SCHEMAP_SUPERNUMEROUS_LIST_ITEM_TYPE", 
"XML_SCHEMAP_INVALID_ATTR_COMBINATION", "XML_SCHEMAP_INVALID_ATTR_INLINE_COMBINATION", 
"XML_SCHEMAP_MISSING_SIMPLETYPE_CHILD", "XML_SCHEMAP_INVALID_ATTR_NAME", 
"XML_SCHEMAP_REF_AND_CONTENT", "XML_SCHEMAP_CT_PROPS_CORRECT_1", 
"XML_SCHEMAP_CT_PROPS_CORRECT_2", "XML_SCHEMAP_CT_PROPS_CORRECT_3", 
"XML_SCHEMAP_CT_PROPS_CORRECT_4", "XML_SCHEMAP_CT_PROPS_CORRECT_5", 
"XML_SCHEMAP_DERIVATION_OK_RESTRICTION_1", "XML_SCHEMAP_DERIVATION_OK_RESTRICTION_2_1_1", 
"XML_SCHEMAP_DERIVATION_OK_RESTRICTION_2_1_2", "XML_SCHEMAP_DERIVATION_OK_RESTRICTION_2_2", 
"XML_SCHEMAP_DERIVATION_OK_RESTRICTION_3", "XML_SCHEMAP_WILDCARD_INVALID_NS_MEMBER", 
"XML_SCHEMAP_INTERSECTION_NOT_EXPRESSIBLE", "XML_SCHEMAP_UNION_NOT_EXPRESSIBLE", 
"XML_SCHEMAP_SRC_IMPORT_3_1", "XML_SCHEMAP_SRC_IMPORT_3_2", "XML_SCHEMAP_DERIVATION_OK_RESTRICTION_4_1", 
"XML_SCHEMAP_DERIVATION_OK_RESTRICTION_4_2", "XML_SCHEMAP_DERIVATION_OK_RESTRICTION_4_3", 
"XML_SCHEMAP_COS_CT_EXTENDS_1_3", "XML_SCHEMAV_NOROOT", "XML_SCHEMAV_UNDECLAREDELEM", 
"XML_SCHEMAV_NOTTOPLEVEL", "XML_SCHEMAV_MISSING", "XML_SCHEMAV_WRONGELEM", 
"XML_SCHEMAV_NOTYPE", "XML_SCHEMAV_NOROLLBACK", "XML_SCHEMAV_ISABSTRACT", 
"XML_SCHEMAV_NOTEMPTY", "XML_SCHEMAV_ELEMCONT", "XML_SCHEMAV_HAVEDEFAULT", 
"XML_SCHEMAV_NOTNILLABLE", "XML_SCHEMAV_EXTRACONTENT", "XML_SCHEMAV_INVALIDATTR", 
"XML_SCHEMAV_INVALIDELEM", "XML_SCHEMAV_NOTDETERMINIST", "XML_SCHEMAV_CONSTRUCT", 
"XML_SCHEMAV_INTERNAL", "XML_SCHEMAV_NOTSIMPLE", "XML_SCHEMAV_ATTRUNKNOWN", 
"XML_SCHEMAV_ATTRINVALID", "XML_SCHEMAV_VALUE", "XML_SCHEMAV_FACET", 
"XML_SCHEMAV_CVC_DATATYPE_VALID_1_2_1", "XML_SCHEMAV_CVC_DATATYPE_VALID_1_2_2", 
"XML_SCHEMAV_CVC_DATATYPE_VALID_1_2_3", "XML_SCHEMAV_CVC_TYPE_3_1_1", 
"XML_SCHEMAV_CVC_TYPE_3_1_2", "XML_SCHEMAV_CVC_FACET_VALID", 
"XML_SCHEMAV_CVC_LENGTH_VALID", "XML_SCHEMAV_CVC_MINLENGTH_VALID", 
"XML_SCHEMAV_CVC_MAXLENGTH_VALID", "XML_SCHEMAV_CVC_MININCLUSIVE_VALID", 
"XML_SCHEMAV_CVC_MAXINCLUSIVE_VALID", "XML_SCHEMAV_CVC_MINEXCLUSIVE_VALID", 
"XML_SCHEMAV_CVC_MAXEXCLUSIVE_VALID", "XML_SCHEMAV_CVC_TOTALDIGITS_VALID", 
"XML_SCHEMAV_CVC_FRACTIONDIGITS_VALID", "XML_SCHEMAV_CVC_PATTERN_VALID", 
"XML_SCHEMAV_CVC_ENUMERATION_VALID", "XML_SCHEMAV_CVC_COMPLEX_TYPE_2_1", 
"XML_SCHEMAV_CVC_COMPLEX_TYPE_2_2", "XML_SCHEMAV_CVC_COMPLEX_TYPE_2_3", 
"XML_SCHEMAV_CVC_COMPLEX_TYPE_2_4", "XML_SCHEMAV_CVC_ELT_1", 
"XML_SCHEMAV_CVC_ELT_2", "XML_SCHEMAV_CVC_ELT_3_1", "XML_SCHEMAV_CVC_ELT_3_2_1", 
"XML_SCHEMAV_CVC_ELT_3_2_2", "XML_SCHEMAV_CVC_ELT_4_1", "XML_SCHEMAV_CVC_ELT_4_2", 
"XML_SCHEMAV_CVC_ELT_4_3", "XML_SCHEMAV_CVC_ELT_5_1_1", "XML_SCHEMAV_CVC_ELT_5_1_2", 
"XML_SCHEMAV_CVC_ELT_5_2_1", "XML_SCHEMAV_CVC_ELT_5_2_2_1", "XML_SCHEMAV_CVC_ELT_5_2_2_2_1", 
"XML_SCHEMAV_CVC_ELT_5_2_2_2_2", "XML_SCHEMAV_CVC_ELT_6", "XML_SCHEMAV_CVC_ELT_7", 
"XML_SCHEMAV_CVC_ATTRIBUTE_1", "XML_SCHEMAV_CVC_ATTRIBUTE_2", 
"XML_SCHEMAV_CVC_ATTRIBUTE_3", "XML_SCHEMAV_CVC_ATTRIBUTE_4", 
"XML_SCHEMAV_CVC_COMPLEX_TYPE_3_1", "XML_SCHEMAV_CVC_COMPLEX_TYPE_3_2_1", 
"XML_SCHEMAV_CVC_COMPLEX_TYPE_3_2_2", "XML_SCHEMAV_CVC_COMPLEX_TYPE_4", 
"XML_SCHEMAV_CVC_COMPLEX_TYPE_5_1", "XML_SCHEMAV_CVC_COMPLEX_TYPE_5_2", 
"XML_SCHEMAV_ELEMENT_CONTENT", "XML_SCHEMAV_DOCUMENT_ELEMENT_MISSING", 
"XML_SCHEMAV_CVC_COMPLEX_TYPE_1", "XML_SCHEMAV_CVC_AU", "XML_SCHEMAV_CVC_TYPE_1", 
"XML_SCHEMAV_CVC_TYPE_2", "XML_SCHEMAV_CVC_IDC", "XML_SCHEMAV_CVC_WILDCARD", 
"XML_SCHEMAV_MISC", "XML_XPTR_UNKNOWN_SCHEME", "XML_XPTR_CHILDSEQ_START", 
"XML_XPTR_EVAL_FAILED", "XML_XPTR_EXTRA_OBJECTS", "XML_C14N_CREATE_CTXT", 
"XML_C14N_REQUIRES_UTF8", "XML_C14N_CREATE_STACK", "XML_C14N_INVALID_NODE", 
"XML_C14N_UNKNOW_NODE", "XML_C14N_RELATIVE_NAMESPACE", "XML_FTP_PASV_ANSWER", 
"XML_FTP_EPSV_ANSWER", "XML_FTP_ACCNT", "XML_FTP_URL_SYNTAX", 
"XML_HTTP_URL_SYNTAX", "XML_HTTP_USE_IP", "XML_HTTP_UNKNOWN_HOST", 
"XML_SCHEMAP_SRC_SIMPLE_TYPE_1", "XML_SCHEMAP_SRC_SIMPLE_TYPE_2", 
"XML_SCHEMAP_SRC_SIMPLE_TYPE_3", "XML_SCHEMAP_SRC_SIMPLE_TYPE_4", 
"XML_SCHEMAP_SRC_RESOLVE", "XML_SCHEMAP_SRC_RESTRICTION_BASE_OR_SIMPLETYPE", 
"XML_SCHEMAP_SRC_LIST_ITEMTYPE_OR_SIMPLETYPE", "XML_SCHEMAP_SRC_UNION_MEMBERTYPES_OR_SIMPLETYPES", 
"XML_SCHEMAP_ST_PROPS_CORRECT_1", "XML_SCHEMAP_ST_PROPS_CORRECT_2", 
"XML_SCHEMAP_ST_PROPS_CORRECT_3", "XML_SCHEMAP_COS_ST_RESTRICTS_1_1", 
"XML_SCHEMAP_COS_ST_RESTRICTS_1_2", "XML_SCHEMAP_COS_ST_RESTRICTS_1_3_1", 
"XML_SCHEMAP_COS_ST_RESTRICTS_1_3_2", "XML_SCHEMAP_COS_ST_RESTRICTS_2_1", 
"XML_SCHEMAP_COS_ST_RESTRICTS_2_3_1_1", "XML_SCHEMAP_COS_ST_RESTRICTS_2_3_1_2", 
"XML_SCHEMAP_COS_ST_RESTRICTS_2_3_2_1", "XML_SCHEMAP_COS_ST_RESTRICTS_2_3_2_2", 
"XML_SCHEMAP_COS_ST_RESTRICTS_2_3_2_3", "XML_SCHEMAP_COS_ST_RESTRICTS_2_3_2_4", 
"XML_SCHEMAP_COS_ST_RESTRICTS_2_3_2_5", "XML_SCHEMAP_COS_ST_RESTRICTS_3_1", 
"XML_SCHEMAP_COS_ST_RESTRICTS_3_3_1", "XML_SCHEMAP_COS_ST_RESTRICTS_3_3_1_2", 
"XML_SCHEMAP_COS_ST_RESTRICTS_3_3_2_2", "XML_SCHEMAP_COS_ST_RESTRICTS_3_3_2_1", 
"XML_SCHEMAP_COS_ST_RESTRICTS_3_3_2_3", "XML_SCHEMAP_COS_ST_RESTRICTS_3_3_2_4", 
"XML_SCHEMAP_COS_ST_RESTRICTS_3_3_2_5", "XML_SCHEMAP_COS_ST_DERIVED_OK_2_1", 
"XML_SCHEMAP_COS_ST_DERIVED_OK_2_2", "XML_SCHEMAP_S4S_ELEM_NOT_ALLOWED", 
"XML_SCHEMAP_S4S_ELEM_MISSING", "XML_SCHEMAP_S4S_ATTR_NOT_ALLOWED", 
"XML_SCHEMAP_S4S_ATTR_MISSING", "XML_SCHEMAP_S4S_ATTR_INVALID_VALUE", 
"XML_SCHEMAP_SRC_ELEMENT_1", "XML_SCHEMAP_SRC_ELEMENT_2_1", "XML_SCHEMAP_SRC_ELEMENT_2_2", 
"XML_SCHEMAP_SRC_ELEMENT_3", "XML_SCHEMAP_P_PROPS_CORRECT_1", 
"XML_SCHEMAP_P_PROPS_CORRECT_2_1", "XML_SCHEMAP_P_PROPS_CORRECT_2_2", 
"XML_SCHEMAP_E_PROPS_CORRECT_2", "XML_SCHEMAP_E_PROPS_CORRECT_3", 
"XML_SCHEMAP_E_PROPS_CORRECT_4", "XML_SCHEMAP_E_PROPS_CORRECT_5", 
"XML_SCHEMAP_E_PROPS_CORRECT_6", "XML_SCHEMAP_SRC_INCLUDE", "XML_SCHEMAP_SRC_ATTRIBUTE_1", 
"XML_SCHEMAP_SRC_ATTRIBUTE_2", "XML_SCHEMAP_SRC_ATTRIBUTE_3_1", 
"XML_SCHEMAP_SRC_ATTRIBUTE_3_2", "XML_SCHEMAP_SRC_ATTRIBUTE_4", 
"XML_SCHEMAP_NO_XMLNS", "XML_SCHEMAP_NO_XSI", "XML_SCHEMAP_COS_VALID_DEFAULT_1", 
"XML_SCHEMAP_COS_VALID_DEFAULT_2_1", "XML_SCHEMAP_COS_VALID_DEFAULT_2_2_1", 
"XML_SCHEMAP_COS_VALID_DEFAULT_2_2_2", "XML_SCHEMAP_CVC_SIMPLE_TYPE", 
"XML_SCHEMAP_COS_CT_EXTENDS_1_1", "XML_SCHEMAP_SRC_IMPORT_1_1", 
"XML_SCHEMAP_SRC_IMPORT_1_2", "XML_SCHEMAP_SRC_IMPORT_2", "XML_SCHEMAP_SRC_IMPORT_2_1", 
"XML_SCHEMAP_SRC_IMPORT_2_2", "XML_SCHEMAP_INTERNAL", "XML_SCHEMAP_NOT_DETERMINISTIC", 
"XML_SCHEMAP_SRC_ATTRIBUTE_GROUP_1", "XML_SCHEMAP_SRC_ATTRIBUTE_GROUP_2", 
"XML_SCHEMAP_SRC_ATTRIBUTE_GROUP_3", "XML_SCHEMAP_MG_PROPS_CORRECT_1", 
"XML_SCHEMAP_MG_PROPS_CORRECT_2", "XML_SCHEMAP_SRC_CT_1", "XML_SCHEMAP_DERIVATION_OK_RESTRICTION_2_1_3", 
"XML_SCHEMAP_AU_PROPS_CORRECT_2", "XML_SCHEMAP_A_PROPS_CORRECT_2", 
"XML_SCHEMAP_C_PROPS_CORRECT", "XML_SCHEMAP_SRC_REDEFINE", "XML_SCHEMAP_SRC_IMPORT", 
"XML_SCHEMAP_WARN_SKIP_SCHEMA", "XML_SCHEMAP_WARN_UNLOCATED_SCHEMA", 
"XML_SCHEMAP_WARN_ATTR_REDECL_PROH", "XML_SCHEMAP_WARN_ATTR_POINTLESS_PROH", 
"XML_SCHEMAP_AG_PROPS_CORRECT", "XML_SCHEMAP_COS_CT_EXTENDS_1_2", 
"XML_SCHEMAP_AU_PROPS_CORRECT", "XML_SCHEMAP_A_PROPS_CORRECT_3", 
"XML_SCHEMAP_COS_ALL_LIMITED", "XML_MODULE_OPEN", "XML_MODULE_CLOSE", 
"XML_CHECK_FOUND_ELEMENT", "XML_CHECK_FOUND_ATTRIBUTE", "XML_CHECK_FOUND_TEXT", 
"XML_CHECK_FOUND_CDATA", "XML_CHECK_FOUND_ENTITYREF", "XML_CHECK_FOUND_ENTITY", 
"XML_CHECK_FOUND_PI", "XML_CHECK_FOUND_COMMENT", "XML_CHECK_FOUND_DOCTYPE", 
"XML_CHECK_FOUND_FRAGMENT", "XML_CHECK_FOUND_NOTATION", "XML_CHECK_UNKNOWN_NODE", 
"XML_CHECK_ENTITY_TYPE", "XML_CHECK_NO_PARENT", "XML_CHECK_NO_DOC", 
"XML_CHECK_NO_NAME", "XML_CHECK_NO_ELEM", "XML_CHECK_WRONG_DOC", 
"XML_CHECK_NO_PREV", "XML_CHECK_WRONG_PREV", "XML_CHECK_NO_NEXT", 
"XML_CHECK_WRONG_NEXT", "XML_CHECK_NOT_DTD", "XML_CHECK_NOT_ATTR", 
"XML_CHECK_NOT_ATTR_DECL", "XML_CHECK_NOT_ELEM_DECL", "XML_CHECK_NOT_ENTITY_DECL", 
"XML_CHECK_NOT_NS_DECL", "XML_CHECK_NO_HREF", "XML_CHECK_WRONG_PARENT", 
"XML_CHECK_NS_SCOPE", "XML_CHECK_NS_ANCESTOR", "XML_CHECK_NOT_UTF8", 
"XML_CHECK_NO_DICT", "XML_CHECK_NOT_NCNAME", "XML_CHECK_OUTSIDE_DICT", 
"XML_CHECK_WRONG_NAME", "XML_CHECK_NAME_NOT_NULL", "XML_I18N_NO_NAME", 
"XML_I18N_NO_HANDLER", "XML_I18N_EXCESS_HANDLER", "XML_I18N_CONV_FAILED", 
"XML_I18N_NO_OUTPUT"))
