if(`test -n "-L/usr/local/lib -lxml2 -lpthread -lz -liconv -lm -L/usr/local/lib"`) then

if(${?LD_LIBRARY_PATH}) then
    setenv LD_LIBRARY_PATH ${LD_LIBRARY_PATH}:-L/usr/local/lib -lxml2 -lpthread -lz -liconv -lm -L/usr/local/lib
else
   setenv LD_LIBRARY_PATH -L/usr/local/lib -lxml2 -lpthread -lz -liconv -lm -L/usr/local/lib
endif

endif
